@@ -1,10 +1,11 @@
 use unicode_segmentation::UnicodeSegmentation;
 
-use crate::key::{Key, KeyModifiers};
+use crate::ui::{Key, KeyModifiers};
 
 #[derive(Clone, Debug)]
 pub struct Input {
     content: String,
+    placeholder: Option<String>,
     cursor: usize,
     length: usize,
 }
@@ -23,24 +24,26 @@ impl Input {
     pub fn new() -> Self {
         Self {
             content: String::new(),
+            placeholder: None,
             cursor: 0,
             length: 0,
         }
     }
 
-    #[cfg(test)]
-    pub fn with_content(mut self, content: &str) -> Self {
-        self.content = String::from(content);
-        self.length = content.graphemes(true).count();
-        self.cursor = std::cmp::min(self.cursor, self.length);
+    pub fn new_with(content: &str) -> Self {
+        let len = content.graphemes(true).count();
 
-        self
+        Self {
+            content: String::from(content),
+            placeholder: None,
+            length: len,
+            cursor: len,
+        }
     }
 
-    pub fn reset_with(&mut self, content: &str) {
-        self.content = String::from(content);
-        self.length = content.graphemes(true).count();
-        self.cursor = self.length;
+    pub fn with_placeholder(mut self, placeholder: &str) -> Self {
+        self.placeholder = Some(String::from(placeholder));
+        self
     }
 
     #[cfg(test)]
@@ -56,6 +59,14 @@ impl Input {
         self
     }
 
+    pub fn is_empty(&self) -> bool {
+        self.length == 0
+    }
+
+    pub fn placeholder(&self) -> Option<&str> {
+        self.placeholder.as_deref()
+    }
+
     pub fn handle_key(&mut self, key: Key) -> bool {
         match key {
             Key::Backspace => self.backspace(),
@@ -185,26 +196,27 @@ impl Input {
 
     fn insert(&mut self, c: char) -> bool {
         let at = self.cursor;
-        self.cursor = self.cursor.saturating_add(1);
 
         if at >= self.length {
             self.content.push(c);
-            self.length = self.length.saturating_add(1);
+            if self.update_length() {
+                self.cursor = self.cursor.saturating_add(1);
+            }
             return true;
         }
 
         let mut result = String::new();
-        let mut length = 0;
         for (index, grapheme) in self.content[..].graphemes(true).enumerate() {
-            length += 1;
             if index == at {
-                length += 1;
                 result.push(c);
             }
             result.push_str(grapheme);
         }
-        self.length = length;
+
         self.content = result;
+        if self.update_length() {
+            self.cursor = self.cursor.saturating_add(1);
+        }
 
         true
     }
@@ -249,19 +261,29 @@ impl Input {
 
         dirty
     }
+
+    fn update_length(&mut self) -> bool {
+        let new_len = self.content[..].graphemes(true).count();
+        let old_len = self.length;
+        self.length = new_len;
+
+        new_len != old_len
+    }
 }
 
 #[cfg(test)]
 mod test {
+    use unicode_segmentation::UnicodeSegmentation;
+
     use super::Input;
-    use crate::key::{Key, KeyModifiers};
+    use crate::ui::{Key, KeyModifiers};
 
     #[test]
     fn move_previous_word() {
         let content = "great 🌍, 🍞, 🚗, 1231321📞, 🎉, 🍆xsa232 s2da ake iak eaik";
 
         let assert = |expected, initial| {
-            let mut input = Input::new().with_content(content).with_cursor(initial);
+            let mut input = Input::new_with(content).with_cursor(initial);
 
             let dirty = input.handle_key(Key::Left(KeyModifiers::CONTROL));
             assert_eq!(expected != initial, dirty,
@@ -297,4 +319,48 @@ mod test {
             assert(49, i);
         }
     }
+
+    #[test]
+    // https://github.com/mikaelmello/inquire/issues/5
+    fn regression_issue_5() {
+        let heart = '♥';
+        let vs16 = '\u{fe0f}';
+
+        let heart_without_vs16 = String::from(heart);
+        let heart_with_vs16 = "♥️";
+        // let heart_with_vs16_char = '♥️'; // this doesn't compile because there are 2 chars
+        let built_heart_with_vs16 = {
+            let mut s = String::from(heart);
+            s.push(vs16);
+            s
+        };
+
+        assert_eq!(6, heart_with_vs16.len());
+        assert_eq!(2, heart_with_vs16.chars().count());
+        assert_eq!(1, heart_with_vs16.graphemes(true).count());
+        assert_eq!(&built_heart_with_vs16, heart_with_vs16);
+
+        let mut input = Input::new();
+        assert_eq!(0, input.length);
+        assert_eq!(0, input.cursor);
+        assert_eq!("", input.content);
+
+        input.insert(heart);
+        {
+            assert_eq!(1, input.length);
+            assert_eq!(1, input.cursor);
+            assert_eq!(heart_without_vs16, input.content);
+            assert_ne!(heart_with_vs16, input.content);
+            assert!(input.content.find(vs16).is_none());
+        }
+
+        input.insert(vs16);
+        {
+            assert_eq!(1, input.length);
+            assert_eq!(1, input.cursor);
+            assert_ne!(heart_without_vs16, input.content);
+            assert_eq!(heart_with_vs16, input.content);
+            assert!(input.content.find(vs16).is_some());
+        }
+    }
 }
