@@ -5,15 +5,42 @@ use crate::{
     error::{InquireError, InquireResult},
     formatter::{self, MultiOptionFormatter},
     input::Input,
-    key::{Key, KeyModifiers},
     option_answer::OptionAnswer,
-    renderer::Renderer,
-    terminal::Terminal,
+    ui::{
+        crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, MultiSelectBackend, RenderConfig,
+    },
     utils::paginate,
     validator::MultiOptionValidator,
 };
 
-/// Selection of any amount of options from an interactive list.
+/// Prompt suitable for when you need the user to select many options (including none if applicable) among a list of them.
+///
+/// The user can select (or deselect) the current highlighted option by pressing space, clean all selections by pressing the left arrow and select all options by pressing the right arrow.
+///
+/// This prompt requires a prompt message and a **non-empty** list of options to be displayed to the user. If the list is empty, the prompt operation will fail with an [`InquireError::InvalidConfiguration`] error.
+///
+/// The options are paginated in order to provide a smooth experience to the user, with the default page size being 7. The user can move from the options and the pages will be updated accordingly, including moving from the last to the first options (or vice-versa).
+///
+/// Customizable options:
+///
+/// - **Prompt message**: Required when creating the prompt.
+/// - **Options list**: Options displayed to the user. Must be **non-empty**.
+/// - **Default selections**: Options that are selected by default when the prompt is first rendered. The user can unselect them. If any of the indices is out-of-range of the option list, the prompt will fail with an [`InquireError::InvalidConfiguration`] error.
+/// - **Starting cursor**: Index of the cursor when the prompt is first rendered. Default is 0 (first option). If the index is out-of-range of the option list, the prompt will fail with an [`InquireError::InvalidConfiguration`] error.
+/// - **Help message**: Message displayed at the line below the prompt.
+/// - **Formatter**: Custom formatter in case you need to pre-process the user input before showing it as the final answer.
+///   - Prints the selected options string value, joined using a comma as the separator, by default.
+/// - **Validator**: Custom validator to make sure a given submitted input pass the specified requirements, e.g. not allowing 0 selected options or limiting the number of options that the user is allowed to select.
+///   - No validators are on by default.
+/// - **Page size**: Number of options displayed at once, 7 by default.
+/// - **Filter function**: Function that defines if an option is displayed or not based on the current filter input.
+/// - **Keep filter flag**: Whether the current filter input should be cleared or not after a selection is made. Defaults to true.
+///
+/// # Example
+///
+/// For a full-featured example, check the [GitHub repository](https://github.com/mikaelmello/inquire/blob/main/examples/multiselect.rs).
+///
+/// [`InquireError::InvalidConfiguration`]: crate::error::InquireError::InvalidConfiguration
 #[derive(Copy, Clone)]
 pub struct MultiSelect<'a> {
     /// Message to be presented to the user.
@@ -52,6 +79,9 @@ pub struct MultiSelect<'a> {
     ///
     /// In case of error, the message is displayed one line above the prompt.
     pub validator: Option<MultiOptionValidator<'a>>,
+
+    /// RenderConfig to apply to the rendered interface.
+    pub render_config: &'a RenderConfig,
 }
 
 impl<'a> MultiSelect<'a> {
@@ -95,6 +125,7 @@ impl<'a> MultiSelect<'a> {
             filter: Self::DEFAULT_FILTER,
             formatter: Self::DEFAULT_FORMATTER,
             validator: Self::DEFAULT_VALIDATOR,
+            render_config: RenderConfig::default_static_ref(),
         }
     }
 
@@ -162,19 +193,25 @@ impl<'a> MultiSelect<'a> {
         self
     }
 
+    /// Sets the provided color theme to this prompt.
+    pub fn with_render_config(mut self, render_config: &'a RenderConfig) -> Self {
+        self.render_config = render_config;
+        self
+    }
+
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     pub fn prompt(self) -> InquireResult<Vec<OptionAnswer>> {
-        let terminal = Terminal::new()?;
-        let mut renderer = Renderer::new(terminal)?;
-        self.prompt_with_renderer(&mut renderer)
+        let terminal = CrosstermTerminal::new()?;
+        let mut backend = Backend::new(terminal, self.render_config)?;
+        self.prompt_with_backend(&mut backend)
     }
 
-    pub(in crate) fn prompt_with_renderer(
+    pub(in crate) fn prompt_with_backend<B: MultiSelectBackend>(
         self,
-        renderer: &mut Renderer,
+        backend: &mut B,
     ) -> InquireResult<Vec<OptionAnswer>> {
-        MultiSelectPrompt::new(self)?.prompt(renderer)
+        MultiSelectPrompt::new(self)?.prompt(backend)
     }
 }
 
@@ -337,16 +374,16 @@ impl<'a> MultiSelectPrompt<'a> {
         return Ok(selected_options);
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
+    fn render<B: MultiSelectBackend>(&mut self, backend: &mut B) -> InquireResult<()> {
         let prompt = &self.message;
 
-        renderer.reset_prompt()?;
+        backend.frame_setup()?;
 
         if let Some(err) = &self.error {
-            renderer.print_error_message(err)?;
+            backend.render_error_message(err)?;
         }
 
-        renderer.print_prompt_input(&prompt, None, &self.input)?;
+        backend.render_multiselect_prompt(&prompt, &self.input)?;
 
         let choices = self
             .filtered_options
@@ -355,32 +392,35 @@ impl<'a> MultiSelectPrompt<'a> {
             .map(|i| OptionAnswer::new(i, self.options.get(i).unwrap()))
             .collect::<Vec<OptionAnswer>>();
 
-        let (paginated_opts, rel_sel) = paginate(self.page_size, &choices, self.cursor_index);
+        let page = paginate(self.page_size, &choices, self.cursor_index);
 
-        for (idx, opt) in paginated_opts.iter().enumerate() {
-            renderer.print_multi_option(
-                rel_sel == idx,
-                self.checked.contains(&opt.index),
+        for (idx, opt) in page.content.iter().enumerate() {
+            backend.render_option(
                 &opt.value,
+                page.selection == idx,
+                self.checked.contains(&opt.index),
             )?;
         }
 
         if let Some(help_message) = self.help_message {
-            renderer.print_help(help_message)?;
+            backend.render_help_message(help_message)?;
         }
 
-        renderer.flush()?;
+        backend.frame_finish()?;
 
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<Vec<OptionAnswer>> {
+    fn prompt<B: MultiSelectBackend>(
+        mut self,
+        backend: &mut B,
+    ) -> InquireResult<Vec<OptionAnswer>> {
         let final_answer: Vec<OptionAnswer>;
 
         loop {
-            self.render(renderer)?;
+            self.render(backend)?;
 
-            let key = renderer.read_key()?;
+            let key = backend.read_key()?;
 
             match key {
                 Key::Cancel => return Err(InquireError::OperationCanceled),
@@ -397,7 +437,7 @@ impl<'a> MultiSelectPrompt<'a> {
 
         let formatted = (self.formatter)(&final_answer);
 
-        renderer.cleanup(&self.message, &formatted)?;
+        backend.finish_prompt(&self.message, &formatted)?;
 
         Ok(final_answer)
     }
