@@ -5,14 +5,50 @@ use crate::{
     error::{InquireError, InquireResult},
     formatter::{self, OptionFormatter},
     input::Input,
-    key::{Key, KeyModifiers},
     option_answer::OptionAnswer,
-    renderer::Renderer,
-    terminal::Terminal,
+    ui::{crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, RenderConfig, SelectBackend},
     utils::paginate,
 };
 
-/// Selection of one option from an interactive list.
+/// Prompt suitable for when you need the user to select one option among many.
+///
+/// The user can select and submit the current highlighted option by pressing space or enter.
+///
+/// This prompt requires a prompt message and a **non-empty** list of options to be displayed to the user. If the list is empty, the prompt operation will fail with an [`InquireError::InvalidConfiguration`] error.
+///
+/// This prompt does not support custom validators because of its nature. A submission always selects exactly one of the options. If this option was not supposed to be selected or is invalid in some way, it probably should not be included in the options list.
+///
+/// The options are paginated in order to provide a smooth experience to the user, with the default page size being 7. The user can move from the options and the pages will be updated accordingly, including moving from the last to the first options (or vice-versa).
+///
+/// Like all others, this prompt also allows you to customize several aspects of it:
+///
+/// - **Prompt message**: Required when creating the prompt.
+/// - **Options list**: Options displayed to the user. Must be **non-empty**.
+/// - **Starting cursor**: Index of the cursor when the prompt is first rendered. Default is 0 (first option). If the index is out-of-range of the option list, the prompt will fail with an [`InquireError::InvalidConfiguration`] error.
+/// - **Help message**: Message displayed at the line below the prompt.
+/// - **Formatter**: Custom formatter in case you need to pre-process the user input before showing it as the final answer.
+///   - Prints the selected option string value by default.
+/// - **Page size**: Number of options displayed at once, 7 by default.
+/// - **Filter function**: Function that defines if an option is displayed or not based on the current filter input.
+///
+/// # Example
+///
+/// ```no_run
+/// use inquire::Select;
+///
+/// let options = vec!["Banana", "Apple", "Strawberry", "Grapes",
+///     "Lemon", "Tangerine", "Watermelon", "Orange", "Pear", "Avocado", "Pineapple",
+/// ];
+///
+/// let ans = Select::new("What's your favorite fruit?", &options).prompt();
+///
+/// match ans {
+///     Ok(choice) => println!("{}! That's mine too!", choice.value),
+///     Err(_) => println!("There was an error, please try again"),
+/// }
+/// ```
+///
+/// [`InquireError::InvalidConfiguration`]: crate::error::InquireError::InvalidConfiguration
 #[derive(Copy, Clone)]
 pub struct Select<'a> {
     /// Message to be presented to the user.
@@ -40,6 +76,9 @@ pub struct Select<'a> {
 
     /// Function that formats the user input and presents it to the user as the final rendering of the prompt.
     pub formatter: OptionFormatter<'a>,
+
+    /// RenderConfig to apply to the rendered interface.
+    pub render_config: &'a RenderConfig,
 }
 
 impl<'a> Select<'a> {
@@ -73,6 +112,7 @@ impl<'a> Select<'a> {
             starting_cursor: Self::DEFAULT_STARTING_CURSOR,
             filter: Self::DEFAULT_FILTER,
             formatter: Self::DEFAULT_FORMATTER,
+            render_config: RenderConfig::default_static_ref(),
         }
     }
 
@@ -118,19 +158,25 @@ impl<'a> Select<'a> {
         self
     }
 
+    /// Sets the provided color theme to this prompt.
+    pub fn with_render_config(mut self, render_config: &'a RenderConfig) -> Self {
+        self.render_config = render_config;
+        self
+    }
+
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     pub fn prompt(self) -> InquireResult<OptionAnswer> {
-        let terminal = Terminal::new()?;
-        let mut renderer = Renderer::new(terminal)?;
-        self.prompt_with_renderer(&mut renderer)
+        let terminal = CrosstermTerminal::new()?;
+        let mut backend = Backend::new(terminal, self.render_config)?;
+        self.prompt_with_backend(&mut backend)
     }
 
-    pub(in crate) fn prompt_with_renderer(
+    pub(in crate) fn prompt_with_backend<B: SelectBackend>(
         self,
-        renderer: &mut Renderer,
+        backend: &mut B,
     ) -> InquireResult<OptionAnswer> {
-        SelectPrompt::new(self)?.prompt(renderer)
+        SelectPrompt::new(self)?.prompt(backend)
     }
 }
 
@@ -230,12 +276,12 @@ impl<'a> SelectPrompt<'a> {
             .and_then(|i| self.options.get(*i).map(|opt| OptionAnswer::new(*i, opt)))
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
+    fn render<B: SelectBackend>(&mut self, backend: &mut B) -> InquireResult<()> {
         let prompt = &self.message;
 
-        renderer.reset_prompt()?;
+        backend.frame_setup()?;
 
-        renderer.print_prompt_input(&prompt, None, &self.input)?;
+        backend.render_select_prompt(&prompt, &self.input)?;
 
         let choices = self
             .filtered_options
@@ -244,28 +290,28 @@ impl<'a> SelectPrompt<'a> {
             .map(|i| OptionAnswer::new(i, self.options.get(i).unwrap()))
             .collect::<Vec<OptionAnswer>>();
 
-        let (paginated_opts, rel_sel) = paginate(self.page_size, &choices, self.cursor_index);
+        let page = paginate(self.page_size, &choices, self.cursor_index);
 
-        for (idx, opt) in paginated_opts.iter().enumerate() {
-            renderer.print_option(rel_sel == idx, &opt.value)?;
+        for (idx, opt) in page.content.iter().enumerate() {
+            backend.render_option(&opt.value, page.selection == idx)?;
         }
 
         if let Some(help_message) = self.help_message {
-            renderer.print_help(help_message)?;
+            backend.render_help_message(help_message)?;
         }
 
-        renderer.flush()?;
+        backend.frame_finish()?;
 
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<OptionAnswer> {
+    fn prompt<B: SelectBackend>(mut self, backend: &mut B) -> InquireResult<OptionAnswer> {
         let final_answer: OptionAnswer;
 
         loop {
-            self.render(renderer)?;
+            self.render(backend)?;
 
-            let key = renderer.read_key()?;
+            let key = backend.read_key()?;
 
             match key {
                 Key::Cancel => return Err(InquireError::OperationCanceled),
@@ -282,7 +328,7 @@ impl<'a> SelectPrompt<'a> {
 
         let formatted = (self.formatter)(&final_answer);
 
-        renderer.cleanup(&self.message, &formatted)?;
+        backend.finish_prompt(&self.message, &formatted)?;
 
         Ok(final_answer)
     }
