@@ -5,17 +5,59 @@ use crate::{
     error::{InquireError, InquireResult},
     formatter::{StringFormatter, DEFAULT_STRING_FORMATTER},
     input::Input,
-    key::{Key, KeyModifiers},
     option_answer::OptionAnswer,
-    renderer::Renderer,
-    terminal::Terminal,
+    ui::{crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, RenderConfig, TextBackend},
     utils::paginate,
     validator::StringValidator,
 };
 
 const DEFAULT_HELP_MESSAGE: &str = "↑↓ to move, tab to auto-complete, enter to submit";
 
-/// Prompts the user for a single line of text input.
+/// Standard text prompt that returns the user string input.
+///
+/// This is the standard the standard kind of prompt you would expect from a library like this one. It displays a message to the user, prompting them to type something back. The user's input is then stored in a `String` and returned to the prompt caller.
+///
+///
+/// ## Configuration options
+///
+/// - **Prompt message**: Main message when prompting the user for input, `"What is your name?"` in the example below.
+/// - **Help message**: Message displayed at the line below the prompt.
+/// - **Default value**: Default value returned when the user submits an empty response.
+/// - **Placeholder**: Short hint that describes the expected value of the input.
+/// - **Validators**: Custom validators to the user's input, displaying an error message if the input does not pass the requirements.
+/// - **Formatter**: Custom formatter in case you need to pre-process the user input before showing it as the final answer.
+/// - **Suggester**: Custom function that returns a list of input suggestions based on the current text input. See more on "Autocomplete" below.
+///
+/// ## Default behaviors
+///
+/// Default behaviors for each one of `Text` configuration options:
+///
+/// - The input formatter just echoes back the given input.
+/// - No validators are called, accepting any sort of input including empty ones.
+/// - No default values or help messages.
+/// - No auto-completion features set-up.
+/// - Prompt messages are always required when instantiating via `new()`.
+///
+/// ## Autocomplete
+///
+/// With `Text` inputs, it is also possible to set-up an auto-completion system to provide a better UX when necessary.
+///
+/// You can set-up a custom [`Suggester`](crate::config::Suggester) function, which receives the current input as the only argument and should return a vector of strings, the suggested values.
+///
+/// The user is then able to select one of them by moving up and down the list, possibly further modifying a selected suggestion.
+///
+/// # Example
+///
+/// ```no_run
+/// use inquire::Text;
+///
+/// let name = Text::new("What is your name?").prompt();
+///
+/// match name {
+///     Ok(name) => println!("Hello {}", name),
+///     Err(_) => println!("An error happened when asking for your name, try again later."),
+/// }
+/// ```
 #[derive(Clone)]
 pub struct Text<'a> {
     /// Message to be presented to the user.
@@ -24,6 +66,9 @@ pub struct Text<'a> {
     /// Default value, returned when the user input is empty.
     pub default: Option<&'a str>,
 
+    /// Short hint that describes the expected value of the input.
+    pub placeholder: Option<&'a str>,
+
     /// Help message to be presented to the user.
     pub help_message: Option<&'a str>,
 
@@ -43,6 +88,9 @@ pub struct Text<'a> {
 
     /// Function that provides a list of suggestions to the user based on the current input.
     pub suggester: Option<Suggester<'a>>,
+
+    /// RenderConfig to apply to the rendered interface.
+    pub render_config: &'a RenderConfig,
 }
 
 impl<'a> Text<'a> {
@@ -62,12 +110,14 @@ impl<'a> Text<'a> {
     pub fn new(message: &'a str) -> Self {
         Self {
             message,
+            placeholder: None,
             default: None,
             help_message: Self::DEFAULT_HELP_MESSAGE,
             validators: Self::DEFAULT_VALIDATORS,
             formatter: Self::DEFAULT_FORMATTER,
             page_size: Self::DEFAULT_PAGE_SIZE,
             suggester: None,
+            render_config: RenderConfig::default_static_ref(),
         }
     }
 
@@ -83,6 +133,12 @@ impl<'a> Text<'a> {
         self
     }
 
+    /// Sets the placeholder.
+    pub fn with_placeholder(mut self, placeholder: &'a str) -> Self {
+        self.placeholder = Some(placeholder);
+        self
+    }
+
     /// Sets the suggester.
     pub fn with_suggester(mut self, suggester: Suggester<'a>) -> Self {
         self.suggester = Some(suggester);
@@ -129,16 +185,25 @@ impl<'a> Text<'a> {
         self
     }
 
+    /// Sets the provided color theme to this prompt.
+    pub fn with_render_config(mut self, render_config: &'a RenderConfig) -> Self {
+        self.render_config = render_config;
+        self
+    }
+
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     pub fn prompt(self) -> InquireResult<String> {
-        let terminal = Terminal::new()?;
-        let mut renderer = Renderer::new(terminal)?;
-        self.prompt_with_renderer(&mut renderer)
+        let terminal = CrosstermTerminal::new()?;
+        let mut backend = Backend::new(terminal, self.render_config)?;
+        self.prompt_with_backend(&mut backend)
     }
 
-    pub(in crate) fn prompt_with_renderer(self, renderer: &mut Renderer) -> InquireResult<String> {
-        TextPrompt::from(self).prompt(renderer)
+    pub(in crate) fn prompt_with_backend<B: TextBackend>(
+        self,
+        backend: &mut B,
+    ) -> InquireResult<String> {
+        TextPrompt::from(self).prompt(backend)
     }
 }
 
@@ -166,7 +231,10 @@ impl<'a> From<Text<'a>> for TextPrompt<'a> {
             formatter: so.formatter,
             validators: so.validators,
             suggester: so.suggester,
-            input: Input::new(),
+            input: so
+                .placeholder
+                .map(|p| Input::new().with_placeholder(p))
+                .unwrap_or_else(|| Input::new()),
             original_input: None,
             error: None,
             cursor_index: 0,
@@ -239,7 +307,7 @@ impl<'a> TextPrompt<'a> {
                 if self.original_input.is_none() {
                     self.original_input = Some(self.input.clone());
                 }
-                self.input.reset_with(suggestion);
+                self.input = Input::new_with(suggestion);
             }
         }
     }
@@ -262,16 +330,16 @@ impl<'a> TextPrompt<'a> {
         Ok(self.input.content().into())
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
+    fn render<B: TextBackend>(&mut self, backend: &mut B) -> InquireResult<()> {
         let prompt = &self.message;
 
-        renderer.reset_prompt()?;
+        backend.frame_setup()?;
 
         if let Some(err) = &self.error {
-            renderer.print_error_message(err)?;
+            backend.render_error_message(err)?;
         }
 
-        renderer.print_prompt_input(&prompt, self.default, &self.input)?;
+        backend.render_prompt(&prompt, self.default, &self.input)?;
 
         let choices = self
             .suggested_options
@@ -282,30 +350,30 @@ impl<'a> TextPrompt<'a> {
 
         let display_cursor = self.cursor_index > 0;
         let list_index = self.cursor_index.saturating_sub(1);
-        let (paginated_opts, rel_sel) = paginate(self.page_size, &choices, list_index);
+        let page = paginate(self.page_size, &choices, list_index);
 
-        for (idx, opt) in paginated_opts.iter().enumerate() {
-            renderer.print_option(display_cursor && rel_sel == idx, &opt.value)?;
+        for (idx, opt) in page.content.iter().enumerate() {
+            backend.render_suggestion(&opt.value, display_cursor && page.selection == idx)?;
         }
 
         if let Some(message) = self.help_message {
-            renderer.print_help(message)?;
+            backend.render_help_message(message)?;
         } else if !choices.is_empty() {
-            renderer.print_help(DEFAULT_HELP_MESSAGE)?;
+            backend.render_help_message(DEFAULT_HELP_MESSAGE)?;
         }
 
-        renderer.flush()?;
+        backend.frame_finish()?;
 
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<String> {
+    fn prompt<B: TextBackend>(mut self, backend: &mut B) -> InquireResult<String> {
         let final_answer: String;
 
         loop {
-            self.render(renderer)?;
+            self.render(backend)?;
 
-            let key = renderer.read_key()?;
+            let key = backend.read_key()?;
 
             match key {
                 Key::Cancel => return Err(InquireError::OperationCanceled),
@@ -320,7 +388,7 @@ impl<'a> TextPrompt<'a> {
             }
         }
 
-        renderer.cleanup(&self.message, &(self.formatter)(&final_answer))?;
+        backend.finish_prompt(&self.message, &(self.formatter)(&final_answer))?;
 
         Ok(final_answer)
     }
@@ -328,13 +396,11 @@ impl<'a> TextPrompt<'a> {
 
 #[cfg(test)]
 mod test {
+    use super::Text;
+    use crate::ui::{crossterm::CrosstermTerminal, Backend, RenderConfig};
     use crossterm::event::{KeyCode, KeyEvent};
     use ntest::timeout;
 
-    use crate::{renderer::Renderer, terminal::Terminal};
-
-    use super::Text;
-
     fn default<'a>() -> Text<'a> {
         Text::new("Question?")
     }
@@ -358,10 +424,12 @@ mod test {
                 let mut read = read.iter();
 
                 let mut write: Vec<u8> = Vec::new();
-                let terminal = Terminal::new_with_io(&mut write, &mut read);
-                let mut renderer = Renderer::new(terminal).unwrap();
 
-                let ans = $prompt.prompt_with_renderer(&mut renderer).unwrap();
+                let terminal = CrosstermTerminal::new_with_io(&mut write, &mut read);
+                let mut backend =
+                    Backend::new(terminal, RenderConfig::default_static_ref()).unwrap();
+
+                let ans = $prompt.prompt_with_backend(&mut backend).unwrap();
 
                 assert_eq!($output, ans);
             }
